# Neo Luminosity & Lightness Mask  Maker 8, 16 & 32 bit Version
# Ver. 0.25c interactive version in developping
# add tranceparency factor
# add luminosity mask
# add Nega mask (2022.5)
# add binded preview image (0.1 / 2022.8)
# coping to control panel of plugin menu (0.1a)
# code cleaning, 32bit image support, UI renewal (0.20) June, 2023
# add auto value scaling (for tranceparent factor), 
#     add No luminosity range masking mode (0.25) Aug, 2023
# (c)2021-23 Ohnishi, Yasuo

from ij import IJ, ImagePlus
from ij import WindowManager as WM
from ij.gui import GenericDialog
from ij.io import OpenDialog
from ij import plugin
from ij.plugin import ChannelSplitter, Duplicator
from ij.plugin.filter import ThresholdToSelection
from ij.process import ImageConverter, ImageProcessor
from ij.process import ByteProcessor, ShortProcessor, FloatProcessor
from loci.plugins import BF
from java.awt.event import AdjustmentListener, ItemListener, TextListener
import os
from org.python.core import codecs
codecs.setDefaultEncoding('utf-8')

#---Convert Value in 8, 16, 32bit mode
def cValue(Value, Mode, L_max):
#     Converting Luminance Value each in 8, 16, 32bit mode
        if Mode == "16-bit":
            Value = Value * 256
        elif Mode == "32-bit":
            Value = Value * (L_max / 255.999)
        else: # 8 or 24
            Value = Value
        return Value
#-----------------
def set_Lmax(imp):
    # set Maximum value of Lightness
    bitDepth = imp.getBitDepth()
    if bitDepth == 32:
        L_max = imp.getDisplayRangeMax()
        if int(L_max) > 65536:
            L_max = 16777215
        elif int(L_max) > 256:
            L_max = 65535
        elif int(L_max) > 1.01:
            L_max = 255
        else:
            L_max = 1.0
    elif bitDepth == 16:
        L_max = 65535
    else:
        L_max = 255
    return L_max

#-----------------
def convertToLuminosity(impOrg, weighted):
    # convert RGB 3 color image to mono Luminosity image
    imp = Duplicator().run(impOrg)
    impChs = ChannelSplitter.split(imp)
    ic = plugin.ImageCalculator()
    if weighted == True:
        impChs[0].getProcessor().multiply(0.299)
        impChs[1].getProcessor().multiply(0.587)
        impChs[2].getProcessor().multiply(0.114)
    else:
        impChs[0].getProcessor().multiply(0.3333)
        impChs[1].getProcessor().multiply(0.3334)
        impChs[2].getProcessor().multiply(0.3333)
    for i in impChs:
        i.getProcessor().setLut(None)
    impTmp1 = ic.run("Add created", impChs[0], impChs[1])
    impLuminosity = ic.run("Add created", impTmp1, impChs[2])
    impTmp1.close()
    imp.close()
    return impLuminosity

#--------------------------------------------
def luminosity_mask_maker(imp1, LTh, HTh, Mode, NegaInvert, Show, TrFac, \
                                                        Lmax):
# imp1: Grayscale image
# LTh: Zone lower Threshold
# NegaInvert: Nega invert True or False
# TrFac: Transparency Factor
# Close Windows
    impGraymask = Duplicator().run(imp1)
    impGraymask.setDisplayRange(0.0, Lmax)
    if Show == "Yes":
        impGraymask.show()
# Make Grayscale Mask Image Files
    V1 = str(LTh) # for threshold expression in file name
# Target (Yellow) Mask Threshold
    V2 = str(cValue(abs(LTh), Mode, Lmax)) # actual Yellow mask Threshold
    IJ.run(impGraymask, "Subtract...","value=" + V2)
    stat = impGraymask.getStatistics()
    print ("impGraymask a: ", stat.min)
    print ("impGraymask a: ", stat.mean)
    print ("impGraymask a: ", stat.max)
    if Mode == "32-bit":
        impGraymask.setDisplayRange(0.0, Lmax)
    if NegaInvert == True: # for making inverted Negative mask
        IJ.run(impGraymask, "Divide...", "value=" + str(TrFac))
    else:
        IJ.run(impGraymask, "Multiply...", "value=" + str(TrFac))
            # making mask more transparent (x 4) < canceled
    if NegaInvert == True: # for making inverted Negative mask
        IJ.run(impGraymask, "Invert","")
        V1 = V1 + "_Nega"
        V = "_Nega_x "+ format(TrFac, '.2f')
    else: # for making positive mask
        V1 = V1 + "_Posi"
        V = "_Posi_x" + format(TrFac, '.2f')
    if Show == "Yes":
        impGraymask.updateAndDraw()  # Show the subtracted image in window
             # Save the Subtracted image x 4 x transparency factor
#   save grayscalee mask
    return impGraymask, V

#-------------------------------------------
def make_binary_mask(impZone, zoneLTh, zoneHTh, Mode, \
                            Show, Invert, Lmax):
    if Invert == False:
        zoneLTh = 0
        # in Positive mask Luminosity mask zone low threshold must be 0!
    ip = impZone.getProcessor()
    if Mode == "8-bit":
        mask = ByteProcessor(ip.width, ip.height)
    if Mode == "16-bit":
        mask = ShortProcessor(ip.width, ip.height)
    else:
        mask = FloatProcessor(ip.width, ip.height)

    mask.setValue(cValue(255, Mode, Lmax))
    if Show == "Yes":
        impZone.show()
    # make Zone mask
    impZone.getProcessor().setThreshold(cValue(zoneLTh, Mode, Lmax),\
         cValue(zoneHTh, Mode, Lmax), ImageProcessor.NO_LUT_UPDATE)
    #print "Zone L & H", zoneLTh, zoneHTh
    roi = ThresholdToSelection.run(impZone)
    mask.setRoi(roi)
    mask.fill(roi)
    try:
        impMask = ImagePlus("Mask", mask)
        impMask.setDisplayRange(0.0, Lmax)
    except: # exception process for ROI missing
        IJ.error("Selected ROI (Region of Interest) is missing.")
        if Show == "Yes":
            impZone.hide()
        impZone.close()
        return
    if Show == "Yes":
        impMask.show() # show Zone mask

    return impMask

#----------------------------------------------
# select base channel for deciding mask upper and lower threshold
def define_impZone(imp0, impR, impG, impB, baseCh):
    if baseCh == "Luminosity":
        impZone = convertToLuminosity(imp0, True) # weighted = True
        chName1 ="Lm"
#        ImageConverter(impZone).convertToRGB()
    elif baseCh == "Lightness":
        impZone = convertToLuminosity(imp0, False) # weighted = False
        chName1 ="L"
#        ImageConverter(impZone).convertToRGB()
    elif baseCh == "Blue":
        impZone = Duplicator().run(impB)
        chName1 ="B"
    elif baseCh == "Red":
        impZone = Duplicator().run(impR)
        chName1 ="R"
    elif baseCh == "Green":
        impZone = Duplicator().run(impG)
        chName1 ="G"
    # if imp0 is not monochrome
    if stackSize != 1:
        impR.close()
        impG.close()
        impB.close()
    imp0.close()
    return impZone, chName1

#------------------------------------
### RGB Manipulation
def RGBmanipulation(imp0, folder,filename, baseCh, parm, colorCh, stackSize):
#  input Parameters
    if parm is not None: # if parm is not null, then get paraneters...
        Mode, YTh, NegaInvert, zoneLTh, zoneHTh, TrFac, Show, NoBinMask = parm
#     Mode: Image Format 16-bit or 8-bit
#     YTh: yellow(or blue) value threshold of mask
#     zoneLTh: lower threshold of mask
#     zoneHTh: upper threshold of mask
#     NegaInvert: make blue mask or not
#     Show: show working windows or not
#     NoBinMask: Do not use binary mask True/False
# Windows Showing Mode
        imp0.show()
    impsCh = ChannelSplitter.split(imp0) #Channel Splitting of imp0
    for i in impsCh: # it needs to convert Grayscale image
        if Mode == "8-bit":
            ImageConverter(i).convertToGray8()
        elif Mode == "16-bit":
            ImageConverter(i).convertToGray16()
        else:
            ImageConverter(i).convertToGray32()
    if stackSize !=1: # if imp0 is not monochrome
        """ R ch."""
        impR =  impsCh[0]
        """ G ch."""
        impG =  impsCh[1]
        """ B ch. & B x 0.3"""
        impB =  impsCh[2]
    # Show Windows
#    imp0.close()
    if Show == "Yes":
        if stackSize !=1: # if imp0 is not monochrome
            impR.show()
            impG.show()
            impB.show()
        imp0.hide()
    if colorCh == "Blue":
        imp1 = Duplicator().run(impB)
        chName0 = "B"
    elif colorCh == "Red":
        imp1 = Duplicator().run(impR)
        chName0 = "R"
    elif colorCh == "Green":
        imp1 = Duplicator().run(impG)
        chName0 = "G"
    elif colorCh == "Lightness": #Lightness or Luminosity
        if stackSize == 1:
            imp1 = Duplicator().run(imp0)
        else:
            imp1 =convertToLuminosity(imp0, False)
        chName0 = "L"
    elif colorCh == "Luminosity": #Lightness or Luminosity
        #IJ.run(imp1, "Conversions...", "weighted")
        imp1 =convertToLuminosity(imp0, True)
        chName0 = "Lm"
    imp1.getProcessor().setLut(None)

    #-----
    # Make Luminosity Mask
    impGrayMask, V = luminosity_mask_maker(imp1, zoneLTh, zoneHTh, Mode, NegaInvert, \
                                            Show, TrFac, L_max)
    if Mode == "32-bit":
        impGrayMask.setDisplayRange(0.0, L_max)
    IJ.saveAsTiff(impGrayMask, folder + filename + "_" + chName0 + \
        "_Mask_Thr_".decode('utf-8') + str(zoneLTh) + "-" + str(zoneHTh) + \
                V.decode('utf-8') + "_x4x" + ". tif".decode('utf-8'))

    if stackSize == 3:
        impZone, chName1 = define_impZone(imp0, impR, impG, impB, baseCh)
    elif stackSize ==1:
        impZone = Duplicator().run(imp0)
        chName1 ="L"
    # impR, G, B are closed
    #-----
    # impZone for Grayscale must be absolutely 8 bit!!
    ImageConverter(impZone).setDoScaling(True)
    if NoBinMask == False: # use binary mask
    # call make_binary_mask
        impMask = make_binary_mask(impZone, zoneLTh, zoneHTh, Mode, \
                        Show, NegaInvert, L_max)
        if Mode == "32-bit":
            impMask.setDisplayRange(0.0, L_max)
    # Save Zone Mask
        IJ.saveAsTiff(impMask, folder + filename +"_" + chName1 + "_Zone" \
            + str(int(zoneLTh)) + "-" + str(int(zoneHTh)) + "_Mask.tif")

    # impMask may be 8bit or 16bit depending on original image bit
    #-----
    # bind Zone mask and Yellow / blue mask
        ic = plugin.ImageCalculator()
        maskBindimp = ic.run("Min created", impGrayMask, impMask) # bind masks
        if Mode == "32-bit":
            maskBindimp.setDisplayRange(0.0, L_max)
        if Show == "Yes":
            maskBindimp.show()
        IJ.saveAsTiff(maskBindimp, folder + filename +"_" + chName0 + "_Zone" + \
            str(int(zoneLTh)) + "-" + str(int(zoneHTh)) + "_" + chName1 + V + \
                                                          "_Mask_Binded.tif")
    # save binded mask
        if Show == "Yes":
            impMask.hide()
            maskBindimp.hide()
        impMask.close()
        maskBindimp.close()

    if Show == "Yes":
        impGrayMask.hide()
        impZone.hide()
    impGrayMask.close()
    impZone.close()


#   End of def RGBmanipulation
#----------------------------------------------
class LuminosityMaskPreviewer(AdjustmentListener, ItemListener, TextListener):
         # previewer for binary mask and observe changes of dialog items
    def __init__(self, impPrv, impPrv1, nfield_upperTh, nfield_lowerTh, C0, C1, \
        nfield_TrFac, checkbox_prv, checkbox_Bmask, checkbox_NoBinMask):
        """
             imp: an ImagePlus
             slider: a java.awt.Scrollbar UI element
             impPrv: canvas imp for preview image
             impPrv1: Original Grayscale image
             nfield_upperTh: upper threshold
             nfield_lowerTh: lower threshold
             preview_checkbox: a java.awt.Checkbox controlling whether to
                                        dynamically update the ImagePlus as the
                                        scrollbar is updated, or not.
             C0, C1: Complementary / Primary mask color name
             nfield_TrFac: Transparency factor a java.awt.TextField
             checkbox_Bmask: Primary color mask? True/False
             checkbox_NoBinMask: No Binary Mask applyed? True/False
        """

        self.impPrv = impPrv
            # main preview image / original image for binary mask
        self.impPrv1 = impPrv1 # original image for grayscale mask
        self.impPrv_binary0 = Duplicator().run(impPrv)
                                            # temporary binary mask image
        self.impPrv_Grayscalemask0 = Duplicator().run(impPrv1)
                                            # temporary grayscale mask image
        self.impPrv.setTitle("luminosity mask preview")
        self.impPrv_binary0.setTitle("binary mask preview")
        self.impPrv_Grayscalemask0.setTitle("grayscale mask preview")
        self.original_ip = impPrv.getProcessor().duplicate()
                                           # original image for binary mask
        self.nfield_upperTh = nfield_upperTh
        self.nfield_lowerTh = nfield_lowerTh
        self.upperThOrg = float(nfield_upperTh.getText())
        self.lowerThOrg = float(nfield_lowerTh.getText())
        self.C0 = C0 # positive mask color name
        self.C1 = C1 # negative mask color name
        self.C = C0  # active mask color name
        self.checkbox_prv = checkbox_prv # Preview checkbox
        self.checkbox_Bmask = checkbox_Bmask # Negative / Positive checkbox
        self.checkbox_NoBinMask = checkbox_NoBinMask # No binary mask checkbox
        self.nfield_TrFac = nfield_TrFac # transparency factor

        self.original_ip = self.impPrv.getProcessor().duplicate()
        self.imp_org = Duplicator().run(self.impPrv)
                                                    # store a copy

    def adjustmentValueChanged(self, event):
        """ event: an AdjustmentEvent with data on the state
                                                of the scroll bar. """
        # if threshold value is changed
        if (IJ.isMacOSX() == False) and event.getValueIsAdjusting():
            # if there are more scrollbar adjustment events queued already
            #                                            > no actions
            return
        upperTh = float(self.nfield_upperTh.getText())
        lowerTh = float(self.nfield_lowerTh.getText())
        # Calculate Auto Lightness Scaling (Ver. 0.25c)
        if lowerTh != self.lowerThOrg or upperTh != self.upperThOrg:
            if upperTh - lowerTh != 0.0:
                NewTrFac = 255.0 / (upperTh - lowerTh)
                self.nfield_TrFac.setText(str(NewTrFac))
                self.lowerThOrg = lowerTh
        preview = self.checkbox_prv.getState()
        if preview: # if preview available
            self.threshold() # make binary mask image

    def textValueChanged(self, event):
        """ event: a textValueChanged with data on the state of
                                                the numeric field. """
        # if Tansparency factor is changed
        try:
            textStrings2 = self.nfield_TrFac.getText()
            self.maskTrFac = float(textStrings2)
                                        # Transparency factor for mask
        except:
            return
        preview = self.checkbox_prv.getState()
        if preview:
            self.threshold()
            # if preview is abailable and parameter is correct, make
            #                                       luminosity mask image
        return

    def itemStateChanged(self, event):
        """ event: an ItemEvent with data on what happened to the \
                                                            checkbox. """
        Bmask = self.checkbox_Bmask.getState()
                # Bmask value: True/False
        if Bmask == True: # make negative mask
            self.C = self.C1
        else: # make positive mask
            self.C = self.C0
        preview = self.checkbox_prv.getState()
        if preview:
            self.threshold() # make grayscale mask image

    def reset(self):
        """ Restore the original ImageProcessor """
        self.impPrv.setProcessor(self.original_ip)

    def threshold(self): # routine for making binary mask
        """ Execute the in-place making binary image of the ImagePlus. """
        ip = self.original_ip
        mask = ip.createProcessor(ip.width, ip.height)
        mask.BLACK_AND_WHITE_LUT
        mask.setValue(255)
        threshold_upper = float(self.nfield_upperTh.getText())
        threshold_lower = float(self.nfield_lowerTh.getText())
        if threshold_upper < threshold_lower:
            threshold_upper = threshold_lower
        if self.C == self.C1: # for making inverted Negative mask
            NegaInvert = True
        else: # for making positive mask
            NegaInvert = False
        maskTrFac =  float(self.nfield_TrFac.getText())
                            # transparency factor for mask
        NoBinMask = self.checkbox_NoBinMask.getState()
        if NoBinMask == False:
            impPrv_binary = make_binary_mask(self.impPrv_binary0, \
                threshold_lower, threshold_upper, "8-bit", "No", \
                                                    NegaInvert, 255)
            ip_tmp = impPrv_binary.getProcessor()
            self.impPrv.setProcessor(ip_tmp) # set combined image for preview
            self.impPrv.setDisplayRange(0, 255) # Attention!
            self.impPrv.updateAndDraw()
        #IJ.showMessage("binary completed")
        impPrv_Grayscalemask, V = \
            luminosity_mask_maker(self.impPrv_Grayscalemask0, \
            threshold_lower, threshold_upper, "8-bit", NegaInvert, "No", \
            maskTrFac, 255)
        impPrv_Grayscalemask.setDisplayRange(0, 255)
        ip_tmp = impPrv_Grayscalemask.getProcessor()
        self.impPrv.setProcessor(ip_tmp) # set combined image for preview
#     combine binary mask imega and grayscale mask image
        if NoBinMask == False:
            ic =plugin.ImageCalculator()
            tmpPrv = ic.run("min created", impPrv_Grayscalemask, \
                                                        impPrv_binary)
            ip_tmpPrv = tmpPrv.getProcessor()
            self.impPrv.setProcessor(ip_tmpPrv) # set combined image for preview
            impPrv_binary.close()

        self.impPrv.getProcessor().setLut (None)
#
        IJ.selectWindow(self.impPrv.getTitle())
#
        impPrv_Grayscalemask.close()

#---------------------------------------------
def thresholdImageUI(imp, imp1, Mode, C0, C1):
    """
      imp: preview image and original for binary mask
      imp1: original for grayscale mask
      C0: positive color name
      C1: negative color name
    """
    WM.getActiveWindow().toFront()
    gd = GenericDialog(C0 +" (" + C1 + ") Mask Maker / " +
                                            "input threshold parameters")
    gd.addCheckbox("Making " + C1 + " Mask", False)
    gd.addCheckbox("No Luminosity Range Masking", False)
    gd.addSlider("Upper Threshold of Mask", 0, 255, 255)
    gd.addSlider("Lower Threshold of Mask", 0, 255, 0)
    gd.addCheckbox("Preview", True)
    gd.addNumericField("Mask Transparency Factor", 1.0, 1)
    showWin = ["Yes","No"]
    gd.addChoice("Show working windows?", showWin, showWin[0])

    # The UI elements for the above two inputs
    slider_upper = gd.getSliders().get(0) # the 1st
    slider_lower = gd.getSliders().get(1) # the 2nd
    nfield_upperTh = gd.getNumericFields().get(0) # NField 0
    nfield_lowerTh = gd.getNumericFields().get(1) #  NField 1
    checkbox_Bmask = gd.getCheckboxes().get(0) #  chkbox for blue mask
    checkbox_NoBinMask = gd.getCheckboxes().get(1) # chkbox for no use of binary mask
    checkbox_prv = gd.getCheckboxes().get(2) # chkbox for preview
        #Attention! Slide Bar also has Numeric Field
    nfield_TrFac = gd.getNumericFields().get(2) # Transparency Factor  NField 2
    choice_showWin = gd.getChoices().get(0)


    if not imp:
        print ("Open an image first!")
        return


    #-- add Luminosity Mask Previewer--
    previewerL = LuminosityMaskPreviewer(imp, imp1, nfield_upperTh, \
        nfield_lowerTh, C0, C1, nfield_TrFac, checkbox_prv, checkbox_Bmask, \
        checkbox_NoBinMask)
    slider_upper.addAdjustmentListener(previewerL)
                                    # set listener for upper slider
    slider_lower.addAdjustmentListener(previewerL)
                                    # set listener for lower slider
    checkbox_Bmask.addItemListener(previewerL)
    checkbox_NoBinMask.addItemListener(previewerL)
    checkbox_prv.addItemListener(previewerL)
                                    # set listener for preview checkbox
    nfield_TrFac.addTextListener(previewerL)
            # set listener for transparency fac. checkbox
#
    gd.showDialog()

    if gd.wasCanceled():
        previewerL.reset()
        Mode = "Stop"
        print ("User canceled dialog!")
    YTh =  0
    NegaInvert = gd.getNextBoolean()
    NoBinMask = gd.getNextBoolean()
    zoneHTh =  gd.getNextNumber()
    zoneLTh =  gd.getNextNumber()
    TrFac =  gd.getNextNumber()
    Show =  gd.getNextChoice()
    imp.hide()
    imp.close()
    return Mode, YTh, NegaInvert, zoneLTh, zoneHTh, TrFac, str(Show), NoBinMask

#-------------------------------------------
class DialogPreviewer(ItemListener):
    def __init__(self, choice1, choice2):
        """
             choice: a java.awt.choice
        """
        self.choice1 = choice1
        self.choice2 = choice2


    def itemStateChanged(self, event):
        """ event: an ItemEvent with data on what happened to
                                                the checkbox. """
        choice_item  = self.choice1.getSelectedItem()
        #print "Dialog Previewr: ", str(choice_item)
        if choice_item == "Luminosity":
            choice_item2 = "Luminosity"
        elif choice_item == "Blue":
            choice_item2 = "Blue"
        elif choice_item == "Green":
            choice_item2 = "Green"
        elif choice_item == "Red":
            choice_item2 = "Red"
        else: # Lightness
            choice_item2 = "Lightness"
        self.choice2.select(choice_item2)
        return

#-----------------------------

def selectChannelUI(impTmpPrv, impprvsCh):
    # dialog of selecting target channel for mask upper & lower threshold
    gd = GenericDialog("Lightness Mask Maker / Target Channel Selection")
    scolorCh = ["Lightness", "Luminosity", "Red","Green","Blue"]
    gd.addChoice("Select Target Channel for Mask", scolorCh, scolorCh[0])
    sbaseCh = ["Lightness", "Luminosity","Red","Green","Blue"]
    gd.addChoice\
        ("Select Target Channel for deciding upper & lower threshold", \
                                                    sbaseCh, sbaseCh[0])

    # The UI elements for the above two inputs
    choice1 = gd.getChoices().get(0)
    choice2 = gd.getChoices().get(1)

    previewer = DialogPreviewer(choice1, choice2)
    choice1.addItemListener(previewer)

    gd.showDialog()

    colorCh =  gd.getNextChoice()
    baseCh =  gd.getNextChoice()
    if gd.wasCanceled():
        colorCh =  "None"
        baseCh =  "None"
    #
    if colorCh == "Blue":
        C0 = "Blue"
        C1 = "Blue inverted"
        impPrv1 = Duplicator().run(impprvsCh[2])
    elif colorCh == "Red":
        C0 = "Red"
        C1 = "Red inverted"
        impPrv1 = Duplicator().run(impprvsCh[0])
    elif colorCh == "Green":
        C0 = "Green"
        C1 = "Green inverted"
        impPrv1 = Duplicator().run(impprvsCh[1])
    elif colorCh == "Luminosity":
        C0 = "Luminosity Positive"
        C1 = "Luminosity Negative"
        if stackSize ==1:
            impPrv1 = Duplicator().run(impTmpPrv)
        else:
            impPrv1 = convertToLuminosity(impTmpPrv, True)
    elif colorCh == "Lightness":
        C0 = "Lightness Positive"
        C1 = "Lightness Negative"
        if stackSize ==1:
            impPrv1 = Duplicator().run(impTmpPrv)
        else:
            impPrv1 = convertToLuminosity(impTmpPrv, False)
    else:
        C0 = "None"
        C1 = "None"
        impPrv1 = Duplicator().run(impTmpPrv)

    ImageConverter(impPrv1).convertToGray8()
    #
    if baseCh != "None": # if process is not canceled
        imp0.hide()
        if baseCh == "Blue":
            impPrv0 = Duplicator().run(impprvsCh[2])
        elif baseCh == "Red":
            impPrv0 = Duplicator().run(impprvsCh[0])
        elif baseCh == "Green":
            impPrv0 = Duplicator().run(impprvsCh[1])
        else:
            impPrv0 = Duplicator().run(impTmpPrv)
        ImageConverter(impPrv0).convertToGray8()
    else:
        impPrv0 = Duplicator().run(impTmpPrv)
    #
    return str(baseCh),  str(colorCh), C0, C1, impPrv0, impPrv1
#------------------------
def load_image(Mode):
    imgCurrent = WM.getCurrentImage()
    if not imgCurrent:
        od = OpenDialog("Choose a file")
        folder = od.getDirectory()
        filename = od.getFileName()
        if filename:
            path = folder + filename
            IJ.log(path)
            IJ.log(folder)
            IJ.log(filename)
    # Always Open file with Bio-format Plugin
            imps = BF.openImagePlus(folder + filename)
                                # Open file with Bio-format Plugin
            imp0 = imps[0] #imp0: Original Image
            imp0.show()
        else:
            Mode = "Stop"
    else: # target image is already open
        imp0 = imgCurrent
        filename = imp0.getTitle()
        try:
            folder = imp0.getOriginalFileInfo().directory
        except:
            folder = IJ.getDirectory("Output_directory")
    return imp0, filename, folder, Mode

#----------main----------
window = WM.getActiveWindow()
Mode = "Auto"
imp0, filename, folder, Mode = load_image(Mode)
L_max = set_Lmax(imp0) # L_max: maximum value of imp0 / global variable
if Mode == "Auto":
    bitdepth = imp0.getBitDepth()
    if bitdepth == 8 or bitdepth == 24:
        Mode = "8-bit"
    elif bitdepth ==16:
        Mode = "16-bit"
    elif bitdepth ==32:
        Mode = "32-bit"
        imp0.setDisplayRange(0.0, L_max)
    else:
        Mode = "16-bit"
stackSize = imp0.getStackSize() # imp0 is color or monochrome?
if bitdepth ==24:
    stackSize =3
if Mode != "Stop": # if user did not cancel execute process
    impprv = Duplicator().run(imp0) # impprv: preview imp
    scale = 1200.0 / impprv.getWidth()
                # preview image scaling value (width = 1200px fixed)
    new_height = impprv.getHeight() * scale # preview image height
    new_height = int(new_height)
    impTmpPrv = impprv.resize(1200, new_height, "none")
    if Mode == "32-bit":
        impTmpPrv.setDisplayRange(0.0, L_max)
    ImageConverter(impTmpPrv).setDoScaling(True) # convert to 8bit RGB
    #if Mode == "32-bit":
        #impTmpPrv.getProcessor().multiply(float(255.0 / L_max))
    impTmpPrv2 = Duplicator().run(impTmpPrv)
    if stackSize ==3:
        #ImageConverter(impTmpPrv).convertToRGB() # convert to 8bit RGB  No use in 32bit
        impprvsCh = ChannelSplitter.split(impTmpPrv)
        for i in impprvsCh:
            ImageConverter(i).convertToGray8()
            BD = i.getBitDepth()
            # IJ.showMessage(str(i) + " impTmpPrv bitdepth: " + str(BD))
        ImageConverter(impTmpPrv).convertToRGB() # convert to 8bit RGB  No use in 32bit
        BD = impTmpPrv.getBitDepth()
        # IJ.showMessage("impTmpPrv bitdepth: " + str(BD))
        baseCh, colorCh, C0, C1, impPrv0, impPrv1 = \
                        selectChannelUI(impTmpPrv, impprvsCh)
        """ select target channel for deciding upper & lower threshold """
    elif stackSize == 1: #imp0 is monochrome
        baseCh = "Lightness"
        colorCh = "Lightness"
        C0 = "Lightness Positive"
        C1 = "Lightness Negative"
        ImageConverter(impTmpPrv).convertToGray8() # convert to 8bit mono
        impPrv1 = Duplicator().run(impTmpPrv)
        impPrv0 = Duplicator().run(impTmpPrv)
    #
    if baseCh != "None": # if process is not canceled
        impPrv0.show()
        tmpBitDepth = impPrv1.getBitDepth()
        # IJ.showMessage("preview bitDepth: " + str(tmpBitDepth))
        parm = thresholdImageUI(impPrv0, impPrv1, Mode, C0, C1)
    else: # if process is  canceled
        parm = []
        parm.append("Stop")
        imp0.hide()
        imp0.close()
    impPrv0.hide()
    impPrv0.close()

    if parm[0] != "Stop":
        RGBmanipulation(imp0, folder, filename, baseCh, parm, \
                                                colorCh, stackSize)
else:
    IJ.log("User canceled")
IJ.log("Finish")

